@@ -306,7 +306,7 @@ run_test_2 (void)
 
   FREEEXPR();
   ADDEXPR ("nothing -z");
-  if (recsel_select (se, test_2_getval, NULL))
+  if (!recsel_select (se, test_2_getval, NULL))
     fail (0, 0);
   FREEEXPR();
   ADDEXPR ("nothing -n");
@@ -334,7 +334,7 @@ run_test_2 (void)
 
   FREEEXPR();
   ADDEXPR ("nothing -f");
-  if (recsel_select (se, test_2_getval, NULL))
+  if (!recsel_select (se, test_2_getval, NULL))
     fail (0, 0);
   FREEEXPR();
   ADDEXPR ("nothing -t");
@@ -369,7 +369,7 @@ run_test_2 (void)
     fail (0, 0);
 
   FREEEXPR();
-  ADDEXPR ("letters -f");
+  ADDEXPR ("letter -f");
   if (!recsel_select (se, test_2_getval, NULL))
     fail (0, 0);
   FREEEXPR();
