@@ -20,10 +20,11 @@
 #include <config.h>
 #include <string.h>
 #include <stdio.h> /* Early versions of bzlib (1.0) require stdio.h */
-#include <bzlib.h>
 
 #include "gpg.h"
-#include "util.h"
+#include "../common/util.h"
+#include <bzlib.h>
+
 #include "packet.h"
 #include "filter.h"
 #include "main.h"
@@ -65,6 +66,9 @@ do_compress(compress_filter_context_t *zfx, bz_stream *bzs, int flush, IOBUF a)
   int zrc;
   unsigned n;
 
+  if (flush == BZ_RUN && bzs->avail_in == 0)
+    return 0;
+
   do
     {
       bzs->next_out = zfx->outbuf;
