@@ -74,14 +74,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <time.h>
 
 #include "scdaemon.h"
 
 #include "../common/i18n.h"
 #include "iso7816.h"
-#include "app-common.h"
 #include "../common/tlv.h"
 
 
@@ -101,7 +99,7 @@ do_learn_status (app_t app, ctrl_t ctrl, unsigned int flags)
 
   /* Return the certificate of the card holder. */
   fid = 0xC000;
-  len = app_help_read_length_of_cert (app->slot, fid, &certoff);
+  len = app_help_read_length_of_cert (app_get_slot (app), fid, &certoff);
   if (!len)
     return 0; /* Card has not been personalized. */
 
@@ -114,7 +112,8 @@ do_learn_status (app_t app, ctrl_t ctrl, unsigned int flags)
 
   /* Now we need to read the certificate, so that we can get the
      public key out of it.  */
-  err = iso7816_read_binary (app->slot, certoff, len-certoff, &der, &derlen);
+  err = iso7816_read_binary (app_get_slot (app), certoff, len-certoff,
+                             &der, &derlen);
   if (err)
     {
       log_info ("error reading entire certificate from FID 0x%04X: %s\n",
@@ -137,7 +136,7 @@ do_learn_status (app_t app, ctrl_t ctrl, unsigned int flags)
       ksba_cert_release (cert);
       return err;
     }
-  err = app_help_get_keygrip_string (cert, hexkeygrip);
+  err = app_help_get_keygrip_string (cert, hexkeygrip, NULL, NULL);
   if (err)
     {
       log_error ("failed to calculate the keygrip for FID 0x%04X\n", fid);
@@ -193,14 +192,14 @@ do_readcert (app_t app, const char *certid,
   /* Read the entire file.  fixme: This could be optimized by first
      reading the header to figure out how long the certificate
      actually is. */
-  err = iso7816_select_file (app->slot, fid, 0);
+  err = iso7816_select_file (app_get_slot (app), fid, 0);
   if (err)
     {
       log_error ("error selecting FID 0x%04X: %s\n", fid, gpg_strerror (err));
       return err;
     }
 
-  err = iso7816_read_binary (app->slot, 0, 0, &buffer, &buflen);
+  err = iso7816_read_binary (app_get_slot (app), 0, 0, &buffer, &buflen);
   if (err)
     {
       log_error ("error reading certificate from FID 0x%04X: %s\n",
@@ -229,7 +228,7 @@ do_readcert (app_t app, const char *certid,
   else
     return gpg_error (GPG_ERR_INV_OBJ);
   totobjlen = objlen + hdrlen;
-  assert (totobjlen <= buflen);
+  log_assert (totobjlen <= buflen);
 
   err = parse_ber_header (&p, &n, &class, &tag, &constructed,
                           &ndef, &objlen, &hdrlen);
@@ -260,7 +259,7 @@ do_readcert (app_t app, const char *certid,
       if ( !(class == CLASS_UNIVERSAL && tag == TAG_SEQUENCE && constructed) )
         return gpg_error (GPG_ERR_INV_OBJ);
       totobjlen = objlen + hdrlen;
-      assert (save_p + totobjlen <= buffer + buflen);
+      log_assert (save_p + totobjlen <= buffer + buflen);
       memmove (buffer, save_p, totobjlen);
     }
 
@@ -293,7 +292,7 @@ verify_pin (app_t app,
   pininfo.maxlen = 8;
 
   if (!opt.disable_pinpad
-      && !iso7816_check_pinpad (app->slot, ISO7816_VERIFY, &pininfo) )
+      && !iso7816_check_pinpad (app_get_slot (app), ISO7816_VERIFY, &pininfo) )
     {
       rc = pincb (pincb_arg,
                   _("||Please enter your PIN at the reader's pinpad"),
@@ -304,7 +303,7 @@ verify_pin (app_t app,
                     gpg_strerror (rc));
           return rc;
         }
-      rc = iso7816_verify_kp (app->slot, 0x81, &pininfo);
+      rc = iso7816_verify_kp (app_get_slot (app), 0x81, &pininfo);
       /* Dismiss the prompt. */
       pincb (pincb_arg, NULL, NULL);
     }
@@ -345,7 +344,8 @@ verify_pin (app_t app,
           return gpg_error (GPG_ERR_BAD_PIN);
         }
 
-      rc = iso7816_verify (app->slot, 0x81, pinvalue, strlen (pinvalue));
+      rc = iso7816_verify (app_get_slot (app), 0x81,
+                           pinvalue, strlen (pinvalue));
       if (gpg_err_code (rc) == GPG_ERR_INV_VALUE)
         {
           /* We assume that ISO 9564-1 encoding is used and we failed
@@ -366,7 +366,8 @@ verify_pin (app_t app,
             paddedpin[i++] = (((*s - '0') << 4) | 0x0f);
           while (i < sizeof paddedpin)
             paddedpin[i++] = 0xff;
-          rc = iso7816_verify (app->slot, 0x81, paddedpin, sizeof paddedpin);
+          rc = iso7816_verify (app_get_slot (app), 0x81,
+                               paddedpin, sizeof paddedpin);
         }
       xfree (pinvalue);
     }
@@ -387,7 +388,7 @@ verify_pin (app_t app,
    that callback should return the PIN in an allocated buffer and
    store that in the 3rd argument.  */
 static gpg_error_t
-do_sign (app_t app, const char *keyidstr, int hashalgo,
+do_sign (app_t app, ctrl_t ctrl, const char *keyidstr, int hashalgo,
          gpg_error_t (*pincb)(void*, const char *, char **),
          void *pincb_arg,
          const void *indata, size_t indatalen,
@@ -409,13 +410,15 @@ do_sign (app_t app, const char *keyidstr, int hashalgo,
                                 + the largest OID _prefix above. */
   int datalen;
 
+  (void)ctrl;
+
   if (!keyidstr || !*keyidstr)
     return gpg_error (GPG_ERR_INV_VALUE);
   if (indatalen != 20 && indatalen != 16 && indatalen != 32
       && indatalen != (15+20) && indatalen != (19+32))
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  /* Check that the provided ID is vaid.  This is not really needed
+  /* Check that the provided ID is valid.  This is not really needed
      but we do it to enforce correct usage by the caller. */
   if (strncmp (keyidstr, "DINSIG.", 7) )
     return gpg_error (GPG_ERR_INV_ID);
@@ -482,7 +485,7 @@ do_sign (app_t app, const char *keyidstr, int hashalgo,
 
   rc = verify_pin (app, pincb, pincb_arg);
   if (!rc)
-    rc = iso7816_compute_ds (app->slot, 0, data, datalen, 0,
+    rc = iso7816_compute_ds (app_get_slot (app), 0, data, datalen, 0,
                              outdata, outdatalen);
   return rc;
 }
@@ -532,7 +535,7 @@ do_change_pin (app_t app, ctrl_t ctrl,  const char *chvnostr,
       return err;
     }
 
-  err = iso7816_change_reference_data (app->slot, 0x81,
+  err = iso7816_change_reference_data (app_get_slot (app), 0x81,
                                        oldpin, oldpinlen,
                                        pinvalue, strlen (pinvalue));
   xfree (pinvalue);
@@ -547,14 +550,16 @@ gpg_error_t
 app_select_dinsig (app_t app)
 {
   static char const aid[] = { 0xD2, 0x76, 0x00, 0x00, 0x66, 0x01 };
-  int slot = app->slot;
+  int slot = app_get_slot (app);
   int rc;
 
   rc = iso7816_select_application (slot, aid, sizeof aid, 0);
   if (!rc)
     {
-      app->apptype = "DINSIG";
+      app->apptype = APPTYPE_DINSIG;
 
+      app->fnc.prep_reselect = NULL;
+      app->fnc.reselect = NULL;
       app->fnc.learn_status = do_learn_status;
       app->fnc.readcert = do_readcert;
       app->fnc.getattr = NULL;
