@@ -77,12 +77,14 @@ enum {
 #define SCM_SCR331      0xe001
 #define SCM_SCR331DI    0x5111
 #define SCM_SCR335      0x5115
+#define SCM_SCR3310     0x5116    /* @MAXX Basic */
 #define SCM_SCR3320     0x5117
 #define SCM_SPR532      0xe003    /* Also used succeeding model SPR332. */
 #define CHERRY_ST2000   0x003e
 #define VASCO_920       0x0920
 #define GEMPC_PINPAD    0x3478
 #define GEMPC_CT30      0x3437
+#define GEMPC_EZIO      0x34c2    /* (!=34c0) Also known as IDBridge CT710 */
 #define VEGA_ALPHA      0x0008
 #define CYBERJACK_GO    0x0504
 #define CRYPTOUCAN      0x81e6
@@ -107,6 +109,15 @@ enum {
 #define CCID_DRIVER_ERR_NO_READER      0x1000c
 #define CCID_DRIVER_ERR_ABORTED        0x1000d
 #define CCID_DRIVER_ERR_NO_PINPAD      0x1000e
+#define CCID_DRIVER_ERR_USB_OTHER      0x10020
+#define CCID_DRIVER_ERR_USB_IO         0x10021
+#define CCID_DRIVER_ERR_USB_ACCESS     0x10023
+#define CCID_DRIVER_ERR_USB_NO_DEVICE  0x10024
+#define CCID_DRIVER_ERR_USB_BUSY       0x10026
+#define CCID_DRIVER_ERR_USB_TIMEOUT    0x10027
+#define CCID_DRIVER_ERR_USB_OVERFLOW   0x10028
+#define CCID_DRIVER_ERR_UI_CANCELLED   0x10030
+#define CCID_DRIVER_ERR_UI_TIMEOUT     0x10031
 
 struct ccid_driver_s;
 typedef struct ccid_driver_s *ccid_driver_t;
@@ -116,12 +127,12 @@ struct ccid_dev_table;
 int ccid_set_debug_level (int level);
 char *ccid_get_reader_list (void);
 
-gpg_error_t ccid_dev_scan (int *idx_max, struct ccid_dev_table **t_p);
-void ccid_dev_scan_finish (struct ccid_dev_table *tbl, int max);
-unsigned int ccid_get_BAI (int, struct ccid_dev_table *tbl);
+gpg_error_t ccid_dev_scan (int *idx_max, void **t_p);
+void ccid_dev_scan_finish (void *tbl0, int max);
+unsigned int ccid_get_BAI (int, void *tbl0);
 int ccid_compare_BAI (ccid_driver_t handle, unsigned int);
 int ccid_open_reader (const char *spec_reader_name,
-                      int idx, struct ccid_dev_table *ccid_table,
+                      int idx, void *ccid_table0,
                       ccid_driver_t *handle, char **rdrname_p);
 int ccid_set_progress_cb (ccid_driver_t handle,
                           void (*cb)(void *, const char *, int, int, int),
